/* Musical Blob
 * University of Washington Computer Science and Engineering
 * Sound Capstone (CSE 481i)
 * Project Members: Nathan Armstrong, Daniel Chesney, Andrew Foster
 *
 * BlobSurfaceDrawer
 * A monolithic and relatively clumsy system for rendering particle systems as
 * a set of metaballs.  Interfaces with Raghavendra Chandrashekarao's code for
 * polygonizing a scalar field using Marching Cubes.
 */

#ifndef __BLOB_ISO_FUNC_HPP__
#define __BLOB_ISO_FUNC_HPP__

#include <vector>
#include "ParticleSystem.hpp"
#include "rchandra/CIsoSurface.h"
#include "ParticleBoundSet.hpp"

using std::vector;

class BlobSurfaceDrawer : public CIsoSurface<double>{
public:
	//Creates a new BlobSurfaceDrawer that uses the given number of cells in
	//the X, Y, and Z axes.  Higher number of cells increases level of detail,
	//but will cause high performance hits.
	BlobSurfaceDrawer(int xCells, int yCells, int zCells);
	virtual ~BlobSurfaceDrawer();

	//Renders the given particle system
	void draw(const ParticleSystem* ps);

protected:
	double* isolevBuffer;
	int xCells, yCells, zCells;

	//Helper functions

	//Evaluates the isolevel of the given eval point with respect to the given
	//source point, using the given multipler (invIsoLev) to scale to a desired
	//range.
	double evalPointIsoLevel(double invIsoLev, const TripleD& source,
			                 const TripleD& evalPoint);

	//Evaluates the isolevel of the given eval point with respect to all source
	//particles in the given vector, using the given multiplier (invIsoLev) to
	//scale to a desired range.
	double sumIsoLevels(const vector<Particle*>& parts,
		TripleD& evalPoint, double invIsoLev);

	//Calculates the isolevels of a grid of points, subdivided by this drawer's
	//cell divisions (assigned on creation), and with size dimensions as given
	//for each axis.  The isolevels are calculated with respect to the source
	//particles in the given vector, and scaled by the given multiplier to meet
	//the desired range.
	void isolevArrGen(const vector<Particle*>& parts,
		TripleD minBounds, double xWidth, double yWidth, double zWidth,
		double isoLev);

	//Generates a polygonization of the surface defined by the given particles
	//using the Metaball technique to define the surface, and using the given
	//bounding points to determine where the surface is to be generated.
	bool generateBlobSurface(const vector<Particle*>& parts,
		TripleD minBounds, TripleD maxBounds);

	//Divides the particles in the given system into small clustered sets that
	//can be rendered together in order to obtain higher resolution, using the
	//given margin to determine the bounding box of each particle.  Returns a
	//list of ParticleBoundSets that represents the partition.
	vector<ParticleBoundSet*>
		partitionParticles(const ParticleSystem* ps, double margin);
};

#endif
