/* Musical Blob
 * University of Washington Computer Science and Engineering
 * Sound Capstone (CSE 481i)
 * Project Members: Nathan Armstrong, Daniel Chesney, Andrew Foster
 *
 * InputManager
 * A monolith class to handler and merge the primary forms of user input and
 * interaction: the Kinect and WiiMotes.
 *
 * Note: This could be made cleaner, and more separable; however, this was the
 * result of a classic all-nighter, added last-minute for proper WiiMote
 * support.
 */

#ifndef __INPUT_MANAGER_HPP__
#define __INPUT_MANAGER_HPP__

#include <map>
#include "ParticleSystem.hpp"
#include "KinectInputHandler.hpp"
#include "LimbTracker.hpp"
#include "WiiManager.hpp"
#include "PointMapper.hpp"

using std::map;

class InputManager{
public:
	//Creates a new InputManager that uses the given ParticleSystem for
	//context, listens for OSCeleton messages on the given port, and maps
	//limb data from kinect-space to program-space using the given PointMapper.
	InputManager(ParticleSystem* ps, unsigned kinectPort,
	             PointMapper mapper);

	//Runs a listening loop on the manager's port.  Does not return.
	void listen();

	//Updates all input systems and stored data appropriately
	void update();

	//Draws representations of user limbs onto the scene
	void drawLimbReps() const;

private:
	class LimbGrasp;

	ParticleSystem* partSys;
	LimbTracker lt;
	KinectInputHandler kih;
	WiiManager wm;
	PointMapper pm;
	map<unsigned, LimbGrasp*> graspMap;

	//Helper functions

	//Adds force to all particles in this manager's particle system based on
	//moving limbs coming into contact with each at the given position, with
	//the given velocity.
	void applyContactForce(TripleD pos, TripleD vel);

	//Adds force to all particles in this manager's particle system, attracting
	//them to the given point (within some cutoff distance)
	void applyAttractForce(TripleD sourcePos);

	//Handles various events and updates for the WiiMote with the given ID
	void handleWiiMoteInput(unsigned mote);

	//Attempts to attach the given WiiMote with a limb if it is not already
	//attached.  The limb attachment will succeed if at least one limb is
	//currently being tracked.  The highest limb will be selected as the
	//attachment limb.
	void testWiiMoteAttachment(unsigned mote);

	//Returns whether the given limb is currently grasping a particle.
	bool isLimbGrasped(unsigned id, LimbType limb) const;

	//Returns whether the given particle is currently being grasped by a limb.
	bool isParticleGrasped(Particle* p) const;
};

#endif
